@@ -16,50 +16,59 @@ import logging
 logger = logging.getLogger(__name__)
 
 WATCH_OPS = {
-    'namespace': 'https://{endpoint}/api/v1/watch/namespaces/{namespace}',
-    'job': 'https://{endpoint}/apis/batch/v1/watch/namespaces/{namespace}/jobs/{name}',  # noqa
+    'namespace': '{scheme}://{endpoint}/api/v1/watch/namespaces/{namespace}',
+    'job': '{scheme}://{endpoint}/apis/batch/v1/watch/namespaces/{namespace}/jobs/{name}',  # noqa
+    'execution': '{scheme}://{endpoint}/api/v1/watch/namespaces/{namespace}/pods/{name}',  # noqa
     'tfjob':
-        'https://{endpoint}/apis/kubeflow.org/v1alpha1/watch/namespaces/{namespace}/tfjobs/{name}'  # noqa
+        '{scheme}://{endpoint}/apis/kubeflow.org/v1alpha1/watch/namespaces/{namespace}/tfjobs/{name}'  # noqa
 }
 
 GET_OPS = {
     'namespace':
-        'https://{endpoint}/api/v1/namespaces/{namespace}',
+        '{scheme}://{endpoint}/api/v1/namespaces/{namespace}',
     'list_jobs':
-        'https://{endpoint}/apis/batch/v1/namespaces/{namespace}/jobs',
+        '{scheme}://{endpoint}/apis/batch/v1/namespaces/{namespace}/jobs',
     'job':
-        'https://{endpoint}/apis/batch/v1/namespaces/{namespace}/jobs/{name}/status',  # noqa
+        '{scheme}://{endpoint}/apis/batch/v1/namespaces/{namespace}/jobs/{name}/status',  # noqa
     'executions':
-        'https://{endpoint}/api/v1/namespaces/{namespace}/pods/?labelSelector=job-name={name}',  # noqa
+        '{scheme}://{endpoint}/api/v1/namespaces/{namespace}/pods/?labelSelector=job-name={name}',  # noqa
     'tfjob':
-        'https://{endpoint}/apis/kubeflow.org/v1alpha1/namespaces/{namespace}/tfjobs/{name}',  # noqa
+        '{scheme}://{endpoint}/apis/kubeflow.org/v1alpha1/namespaces/{namespace}/tfjobs/{name}',  # noqa
     'list_tfjobs':
-        'https://{endpoint}/apis/kubeflow.org/v1alpha1/namespaces/{namespace}/tfjobs',  # noqa
+        '{scheme}://{endpoint}/apis/kubeflow.org/v1alpha1/namespaces/{namespace}/tfjobs',  # noqa
     'log':
-        'https://{endpoint}/api/v1/namespaces/{namespace}/pods/{name}/log',
+        '{scheme}://{endpoint}/api/v1/namespaces/{namespace}/pods/{name}/log',
     'nodes':
-        'https://{endpoint}/api/v1/nodes/',
-    'configmaps': 'https://{endpoint}/api/v1/namespaces/{namespace}/configmaps?{selector}'  # noqa
+        '{scheme}://{endpoint}/api/v1/nodes/',
+    'configmaps': '{scheme}://{endpoint}/api/v1/namespaces/{namespace}/configmaps?{selector}',  # noqa
+    'deployments': '{scheme}://{endpoint}/apis/apps/v1/namespaces/{namespace}/deployments/{name}',  # noqa
+    'scale': '{scheme}://{endpoint}/apis/apps/v1/namespaces/{namespace}/deployments/{name}/scale'  # noqa
+}
+
+PATCH_OPS = {
+    'scale': '{scheme}://{endpoint}/apis/apps/v1/namespaces/{namespace}/deployments/{name}/scale'  # noqa
 }
 
 POST_OPS = {
     'namespace':
-        ('https://{endpoint}/api/v1/namespaces', 'v1'),
+        ('{scheme}://{endpoint}/api/v1/namespaces', 'v1'),
     'job':
-        ('https://{endpoint}/apis/batch/v1/namespaces/{namespace}/jobs', 'batch/v1'),  # noqa
+        ('{scheme}://{endpoint}/apis/batch/v1/namespaces/{namespace}/jobs', 'batch/v1'),  # noqa
     'tfjob':
-        ('https://{endpoint}/apis/kubeflow.org/v1alpha1/namespaces/{namespace}/tfjobs', 'kubeflow.org/v1alpha1'),  # noqa
+        ('{scheme}://{endpoint}/apis/kubeflow.org/v1alpha1/namespaces/{namespace}/tfjobs', 'kubeflow.org/v1alpha1'),  # noqa
     'quota':
-        ('https://{endpoint}/api/v1/namespaces/{namespace}/resourcequotas', 'v1')  # noqa
+        ('{scheme}://{endpoint}/api/v1/namespaces/{namespace}/resourcequotas', 'v1')  # noqa
 }
 
 DELETE_OPS = {
     'namespace':
-        ('https://{endpoint}/api/v1/namespaces/{namespace}', 'v1'),
+        ('{scheme}://{endpoint}/api/v1/namespaces/{namespace}', 'v1'),
+    'execution':
+        ('{scheme}://{endpoint}/api/v1/namespaces/{namespace}/pods/{name}', 'v1'),  # noqa
     'job':
-        ('https://{endpoint}/apis/batch/v1/namespaces/{namespace}/jobs/{name}', 'batch/v1'),  # noqa
+        ('{scheme}://{endpoint}/apis/batch/v1/namespaces/{namespace}/jobs/{name}', 'batch/v1'),  # noqa
     'tfjob':
-        ('https://{endpoint}/apis/kubeflow.org/v1alpha1/namespaces/{namespace}/tfjobs/{name}', 'v1')  # noqa
+        ('{scheme}://{endpoint}/apis/kubeflow.org/v1alpha1/namespaces/{namespace}/tfjobs/{name}', 'v1')  # noqa
 }
 
 
@@ -67,11 +76,13 @@ class K8SCaller(object):
     constants = const
     _type = 'k8s'
 
-    def __init__(self, ssl_context, endpoint, session, verify=True):
+    def __init__(self, ssl_context, endpoint, session,
+                 verify=True, scheme='https'):
         self.ssl_context = ssl_context
         self.endpoint = endpoint
         self.session = session
         self.verify = verify
+        self.scheme = scheme
 
     @property
     def type(self):
@@ -81,7 +92,8 @@ class K8SCaller(object):
         url = GET_OPS['namespace']
         url = url.format(
             namespace=namespace,
-            endpoint=self.endpoint)
+            endpoint=self.endpoint,
+            scheme=self.scheme)
         result = await self.get(url)
         if result is None:
             return None
@@ -92,7 +104,8 @@ class K8SCaller(object):
         url, version = POST_OPS['namespace']
         url = url.format(
             namespace=namespace,
-            endpoint=self.endpoint)
+            endpoint=self.endpoint,
+            scheme=self.scheme)
         obj = K8SNamespace(namespace)
         return await self.post(url, version, obj.payload())
 
@@ -100,7 +113,8 @@ class K8SCaller(object):
         url, version = DELETE_OPS['namespace']
         url = url.format(
             namespace=namespace,
-            endpoint=self.endpoint)
+            endpoint=self.endpoint,
+            scheme=self.scheme)
         obj = K8SDelete()
         return await self.delete(url, version, obj.payload())
 
@@ -109,7 +123,8 @@ class K8SCaller(object):
         url = url.format(
             namespace=namespace,
             name=name,
-            endpoint=self.endpoint)
+            endpoint=self.endpoint,
+            scheme=self.scheme)
         return await self.watch(url, value='ADDED', timeout=timeout)
 
     async def wait_deleted(self, kind, namespace, name=None, timeout=30):
@@ -117,14 +132,16 @@ class K8SCaller(object):
         url = url.format(
             namespace=namespace,
             name=name,
-            endpoint=self.endpoint)
+            endpoint=self.endpoint,
+            scheme=self.scheme)
         return await self.watch(url, value='DELETED', timeout=timeout)
 
     async def define_quota(self, namespace, cpu_limit=None, mem_limit=None):
         url, version = POST_OPS['quota']
         url = url.format(
             namespace=namespace,
-            endpoint=self.endpoint)
+            endpoint=self.endpoint,
+            scheme=self.scheme)
         obj = K8SQuota(
             namespace=namespace,
             max_cpu=cpu_limit,
@@ -134,19 +151,53 @@ class K8SCaller(object):
     async def get_nodes(self):
         url = GET_OPS['nodes']
         url = url.format(
-            endpoint=self.endpoint)
+            endpoint=self.endpoint,
+            scheme=self.scheme)
         result = await self.get(url)
         if result is None:
             return None
         else:
             return K8SNodeList(data=result)
 
+    async def get_scale_deploy(self, namespace, name):
+        url = GET_OPS['scale']
+        url = url.format(
+            namespace=namespace,
+            name=name,
+            endpoint=self.endpoint,
+            scheme=self.scheme)
+        result = await self.get(url)
+        if result is None:
+            return None
+        else:
+            logger.warning(result)
+            return result['status']['replicas']
+
+    async def set_scale_deploy(self, namespace, name, scale):
+        url = PATCH_OPS['scale']
+        url = url.format(
+            namespace=namespace,
+            name=name,
+            endpoint=self.endpoint,
+            scheme=self.scheme)
+        obj = [{
+            "op": "replace",
+            "path": "/spec/replicas",
+            "value": scale
+        }]
+        result = await self.patch(url, obj)
+        if result is None:
+            return None
+        else:
+            return result['spec']['replicas']
+
     async def get_job(self, namespace, name):
         url = GET_OPS['job']
         url = url.format(
             namespace=namespace,
             name=name,
-            endpoint=self.endpoint)
+            endpoint=self.endpoint,
+            scheme=self.scheme)
         result = await self.get(url)
         if result is None:
             return None
@@ -158,7 +209,8 @@ class K8SCaller(object):
         url = url.format(
             namespace=namespace,
             name=name,
-            endpoint=self.endpoint)
+            endpoint=self.endpoint,
+            scheme=self.scheme)
         result = await self.get(url)
         if result is None:
             return None
@@ -170,7 +222,8 @@ class K8SCaller(object):
         url = url.format(
             namespace=namespace,
             name=name,
-            endpoint=self.endpoint)
+            endpoint=self.endpoint,
+            scheme=self.scheme)
         result = await self.get(url)
         if result is None:
             return None
@@ -182,7 +235,8 @@ class K8SCaller(object):
         url = url.format(
             namespace=namespace,
             name=execution_id,
-            endpoint=self.endpoint)
+            endpoint=self.endpoint,
+            scheme=self.scheme)
         result = await self.get(url, noaccept=True)
         if result is None:
             return None
@@ -194,7 +248,8 @@ class K8SCaller(object):
         url = url.format(
             namespace=namespace,
             name=execution_id,
-            endpoint=self.endpoint)
+            endpoint=self.endpoint,
+            scheme=self.scheme)
         async for logline in await self._watch_log(url, timeout=3660):
             yield logline
 
@@ -203,19 +258,36 @@ class K8SCaller(object):
         url = url.format(
             namespace=namespace,
             name=name,
-            endpoint=self.endpoint)
+            endpoint=self.endpoint,
+            scheme=self.scheme)
         obj = K8SDelete()
         await self.delete(url, version, obj.payload())
         if wait:
             return await self.wait_deleted('job', namespace, name)
         return True
 
+    async def delete_execution(self, namespace, job_id,
+                               execution_id, wait=False):
+        url, version = DELETE_OPS['execution']
+        url = url.format(
+            namespace=namespace,
+            name=execution_id,
+            endpoint=self.endpoint,
+            scheme=self.scheme)
+        obj = K8SDelete()
+        await self.delete(url, version, obj.payload())
+        if wait:
+            return await self.wait_deleted(
+                'execution', namespace, execution_id)
+        return True
+
     async def delete_tfjob(self, namespace, name, wait=False):
         url, version = DELETE_OPS['tfjob']
         url = url.format(
             namespace=namespace,
             name=name,
-            endpoint=self.endpoint)
+            endpoint=self.endpoint,
+            scheme=self.scheme)
         obj = K8SDelete()
         await self.delete(url, version, obj.payload())
         if wait:
@@ -226,7 +298,8 @@ class K8SCaller(object):
         url = GET_OPS['list_jobs']
         url = url.format(
             namespace=namespace,
-            endpoint=self.endpoint)
+            endpoint=self.endpoint,
+            scheme=self.scheme)
         result = await self.get(url)
         if result is None:
             return None
@@ -237,7 +310,8 @@ class K8SCaller(object):
         url = GET_OPS['list_tfjobs']
         url = url.format(
             namespace=namespace,
-            endpoint=self.endpoint)
+            endpoint=self.endpoint,
+            scheme=self.scheme)
         result = await self.get(url)
         if result is None:
             return None
@@ -254,7 +328,8 @@ class K8SCaller(object):
         url = url.format(
             namespace=namespace,
             name=name,
-            endpoint=self.endpoint)
+            endpoint=self.endpoint,
+            scheme=self.scheme)
         obj = K8SJob(
             namespace=namespace,
             name=name,
@@ -276,7 +351,8 @@ class K8SCaller(object):
         url = url.format(
             namespace=namespace,
             name=name,
-            endpoint=self.endpoint)
+            endpoint=self.endpoint,
+            scheme=self.scheme)
         obj = K8STFJob(
             namespace=namespace,
             name=name,
@@ -293,7 +369,8 @@ class K8SCaller(object):
         url = url.format(
             namespace=namespace,
             name=name,
-            endpoint=self.endpoint)
+            endpoint=self.endpoint,
+            scheme=self.scheme)
         try:
             # We only wait 1 hour
             async for data in self._watch(url, 3660):
@@ -359,6 +436,23 @@ class K8SCaller(object):
             else:
                 return await resp.json()
 
+    async def patch(self, url, payload):
+        async with self.session.patch(
+                url,
+                json=payload,
+                headers={
+                    'Accept': 'application/json',
+                    'Content-Type': 'application/json-patch+json'
+                },
+                ssl_context=self.ssl_context,
+                verify_ssl=self.verify) as resp:
+            if resp.status in [201, 200]:
+                return await resp.json()
+            else:
+                text = await resp.text()
+                raise Exception(
+                    'Error calling k8s: %d - %s' % (resp.status, text))
+
     async def post(self, url, version, payload):
         payload['apiVersion'] = version
         async with self.session.post(
@@ -373,8 +467,8 @@ class K8SCaller(object):
                 return await resp.json()
             else:
                 text = await resp.text()
-                logger.error('Error calling k8s: %d - %s' % (resp.status, text))  # noqa
-                raise Exception('Error calling k8s')
+                raise Exception(
+                    'Error calling k8s: %d - %s' % (resp.status, text))
 
     async def delete(self, url, version, payload):
         payload['apiVersion'] = version
@@ -394,8 +488,8 @@ class K8SCaller(object):
                 return None
             else:
                 text = await resp.text()
-                logger.error('Error calling k8s: %d - %s' % (resp.status, text))  # noqa
-                raise Exception('Error calling k8s')
+                raise Exception(
+                    'Error calling k8s: %d - %s' % (resp.status, text))
 
     async def get_config_maps(self, namespace, labels=None):
         selector = ''
@@ -407,5 +501,6 @@ class K8SCaller(object):
         url = url.format(
             namespace=namespace,
             endpoint=self.endpoint,
-            selector=selector)
+            selector=selector,
+            scheme=self.scheme)
         return await self.get(url)
