@@ -101,6 +101,10 @@ class ClusterManager:
                 namespace, job_id, execution_id):
             yield log_line
 
+    async def delete_execution(self, namespace, job_id, execution_id):
+        return await self.caller.delete_execution(
+            namespace, job_id, execution_id)
+
     async def wait_for_job_execution_status(self, namespace, name):
         status = self._const.PENDING
         while status not in [self._const.RUNNING, self._const.ERROR,
@@ -133,6 +137,12 @@ class ClusterManager:
                 result.append(key)
         return result
 
+    async def get_scale_deploy(self, namespace, name):
+        return await self.caller.get_scale_deploy(namespace, name)
+
+    async def set_scale_deploy(self, namespace, name, scale):
+        return await self.caller.set_scale_deploy(namespace, name, scale)
+
     # Only K8S
 
     async def install_tfjobs(self, namespace, cloud=None, install_helm=False):
