@@ -14,6 +14,7 @@ logger = logging.getLogger(__name__)
 
 WATCH_OPS = {
     'job': 'http://{endpoint}/v1/job/{namespace}-{name}',
+    'execution': 'http://{endpoint}/v1/job/{namespace}-{name}/allocations/{execution}',  # noqa
     'namespace': None
 }
 
@@ -30,7 +31,8 @@ POST_OPS = {
 }
 
 DELETE_OPS = {
-    'job': 'http://{endpoint}/v1/job/{namespace}-{name}?purge=true'
+    'job': 'http://{endpoint}/v1/job/{namespace}-{name}?purge=true',
+    'execution': 'http://{endpoint}/v1/job/{namespace}-{name}/allocations/{execution}?purge=true',  # noqa
 }
 
 
@@ -79,13 +81,14 @@ class NomadCaller:
             endpoint=self.endpoint)
         return await self.watch(url, not_value=['pending'], timeout=timeout)
 
-    async def wait_deleted(self, kind, namespace, name=None):
+    async def wait_deleted(self, kind, namespace, name=None, execution_id=None):  # noqa
         url = WATCH_OPS[kind]
         if url is None:
             return False
         url = url.format(
             namespace=namespace,
             name=name,
+            execution=execution_id,
             endpoint=self.endpoint)
         return await self.watch(url, value=['dead', 'complete'])
 
@@ -168,6 +171,20 @@ class NomadCaller:
             return await self.wait_deleted('job', namespace, name)
         return True
 
+    async def delete_execution(self, namespace, job_id,
+                               execution_id, wait=False):
+        url = DELETE_OPS['execution']
+        url = url.format(
+            namespace=namespace,
+            name=job_id,
+            execution=execution_id,
+            endpoint=self.endpoint)
+        await self.delete(url, None, None)
+        if wait:
+            return await self.wait_deleted(
+                'execution', namespace, job_id, execution_id)
+        return True
+
     async def delete_tfjob(self, namespace, name, wait=False):
         # TODO
         return None
@@ -294,8 +311,7 @@ class NomadCaller:
                 return await resp.json()
             else:
                 text = await resp.text()
-                logger.error('Error call: %d - %s' % (resp.status, text))
-                raise Exception('Error calling nomad')
+                raise Exception('Error call: %d - %s' % (resp.status, text))
 
     async def delete(self, url, version, payload):
         async with self.session.delete(
@@ -310,8 +326,7 @@ class NomadCaller:
                 return None
             else:
                 text = await resp.text()
-                logger.error('Error call: %d - %s' % (resp.status, text))
-                raise Exception('Error calling nomad')
+                raise Exception('Error call: %d - %s' % (resp.status, text))
 
     async def get_config_maps(self, namespace, labels=None):
-        raise NotImplemented()
+        raise NotImplementedError()
